// Very simple and inaccurate, but it's better than nothing.
float water_fog() {
    float TerrainDepth = texture2D(depthtex1, gl_FragCoord.xy/resolution).x;
    TerrainDepth = linearize_depth(TerrainDepth);
    float ScreenDepth = linearize_depth(gl_FragCoord.z);
    return smoothstep(0, 64, TerrainDepth-ScreenDepth)*WATER_FOG_STRENGTH;
}

float schlick(vec3 V, vec3 N) {
    const float R = 0.1;
    float Theta = 1-dot(-V, N);
    Theta = max(Theta, 0); // https://www.youtube.com/watch?v=BcxKINWMD8M
    return R + (1-R)*(pow(Theta, 5.0));
}

vec3 get_water_normal(vec2 Coords) {
	float Offset = 0.3;
	float D0 = noise_water(vec2(Coords.x+Offset, Coords.y));
	float D1 = noise_water(vec2(Coords.x-Offset, Coords.y));
	float D2 = noise_water(vec2(Coords.x, Coords.y+Offset));
    float D3 = noise_water(vec2(Coords.x, Coords.y-Offset));
    float A = (D0-D1)/Offset;
    float B = (D2-D3)/Offset;
	return normalize(vec3(A, B, 1-(A*A+B*B)));
}

vec3 sky_reflection(vec3 ReflectedVec, float Dist) {
    #ifdef DIMENSION_OVERWORLD
    vec3 Reflection = get_sky(ReflectedVec, Dist);
    Reflection.rgb += round_sun(Dist)*4;

    #ifdef REFLECT_CLOUDS
    vec3 PlayerPos = to_player_pos(ReflectedVec);
    vec3 PlayerPosN = normalize(PlayerPos);
    if(PlayerPos.y > 0) {
			Reflection.rgb += get_stars(PlayerPos);

			#if defined FANCY_CLOUDS
				vec3 CloudPos = PlayerPosN;
				CloudPos.xz = PlayerPos.xz / (PlayerPos.y + length(PlayerPos.xz)/4);
				
				Reflection.rgb = get_clouds(CloudPos, Reflection.rgb);
			#endif
		}
    #endif

    return Reflection;
    #else
    return fogColor.rgb;
    #endif
}

vec3 ssr(vec3 RVec, float Dist, vec3 ViewPos) {
    vec3 ScreenPos = gl_FragCoord.xyz/vec3(resolution, 1);
    // Convert RVec to screenspace
    vec3 Offset = normalize(view_screen(ViewPos+RVec)-ScreenPos)/SSR_STEPS;

    float Noise = ordered_dither(gl_FragCoord.xy);
    vec3 ExpectedPos = ScreenPos+Offset*Noise;
    for(int i = 1; i <= SSR_STEPS; i++) {
        
        // Stretch the reflections a bit horizontally
        if(ExpectedPos.x < -0.1 || ExpectedPos.y < 0 || ExpectedPos.x > 1.1 || ExpectedPos.y > 1) {
            return sky_reflection(RVec, Dist);
        }
        
        float RealDepth = texture2D(depthtex1, ExpectedPos.xy).x;
        if(RealDepth >= 1 || RealDepth < 0.56) {
            return sky_reflection(RVec, Dist);
        }
        
        if(ExpectedPos.z > RealDepth) {
            // Depth based rejection
            if(ExpectedPos.z-RealDepth > Offset.z) break;
            
            // Binary refinement
            for(int i = 1; i <= 3; i++) {
                Offset /= 2;
                vec3 EPos1 = ExpectedPos-Offset;
                float RDepth1 = texture2D(depthtex1, EPos1.xy).x;
                if(EPos1.z > RDepth1) {
                    ExpectedPos = EPos1;
                }
            }
            return texture2D(colortex4, ExpectedPos.xy).rgb;
        }
        ExpectedPos += Offset;
    }
    return sky_reflection(RVec, Dist);
}

vec4 get_fancy_water(vec3 ViewPos, vec4 BaseColor, float SkyBrightness) {
    vec3 ViewPosN = normalize(ViewPos);
    BaseColor.a = min(BaseColor.a+water_fog(), 1);

    vec3 WorldPos = to_player_pos(ViewPos)+cameraPosition;

    vec3 NormalMap = get_water_normal(vec2(WorldPos.x, WorldPos.z));
    vec3 WaterNormal = TBN*NormalMap;

    vec3 ReflectedVec = reflect(ViewPosN, WaterNormal);
    float Dist = distance(ReflectedVec, sunPositionN);

    #ifdef SSR
    vec3 Reflection = ssr(ReflectedVec, Dist, ViewPos);
    #else
    vec3 Reflection = sky_reflection(ReflectedVec, Dist);
    #endif

    BaseColor.rgb = mix(BaseColor.rgb, Reflection, schlick(ViewPosN, WaterNormal)*SkyBrightness);

    vec3 SkyColor = get_sky(ViewPosN, distance(ViewPosN, sunPositionN));
    BaseColor.rgb = get_fog_main(WorldPos-cameraPosition, BaseColor.rgb, gl_FragCoord.z, texcoord, SkyColor);
    return BaseColor;
}
